package de.neusta.imonitor.correlation;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.correlation.configs.CorrelationConfig;
import de.neusta.imonitor.correlation.configs.InputConfig;
import de.neusta.imonitor.correlation.garbagecollection.IndividualGarbageCollector;
import de.neusta.imonitor.correlation.input.IInputProcessor;
import de.neusta.imonitor.correlation.input.InputProcessorFactory;
import de.neusta.imonitor.correlation.ruleloader.RuleLoader;
import de.neusta.imonitor.correlation.services.OntologyInitService;
import de.neusta.imonitor.correlation.threadpool.IndividualGarbageCollectionThreadPool;
import de.neusta.imonitor.correlation.utils.LogUtils;
import de.neusta.imonitor.ontology.Ontology;
import de.neusta.imonitor.ontology.configs.OntologyConfig;
import de.neusta.imonitor.ruleparser.exception.RuleValidationException;

@Service
public class CorrelationStartup {

	private final static Logger LOGGER = LoggerFactory.getLogger(CorrelationStartup.class);

	@Autowired
	private CorrelationConfig correlationConfig;
	@Autowired
	private InputProcessorFactory inputProcessorFactory;
	@Autowired
	private InputConfig inputConfig;
	@Autowired
	private OntologyConfig ontologyConfig;
	@Autowired
	private OntologyInitService ontologyInitService;
	@Autowired
	private RuleLoader ruleLoader;
	@Autowired
	private IndividualGarbageCollector individualGarbageCollector;
	@Autowired
	private IndividualGarbageCollectionThreadPool individualGarbageCollectionThreadPool;
	@Autowired
	private Ontology ontology;

	public void start() throws Exception {
		try {
			LOGGER.info("Starting correlation...");
			if (!checkConfigs()) {
				LOGGER.error("Could not start correlation due to errors in the configs.");
				return;
			}
			this.ontologyInitService.loadAndInitOntology();
			try
			{
				this.ruleLoader.loadRulesFromDatabase(ontology);
			} catch(RuleValidationException ex) {
				LOGGER.error("An error occurred while validating the rules: " + ex.getFailure());
				if (!StringUtils.isEmpty(ex.getQuery())) {
					LOGGER.error("The failing query was: " + ex.getQuery());
				}
				LogUtils.logException(LOGGER, ex);
				return;
			}
			individualGarbageCollectionThreadPool.execute(individualGarbageCollector);
			processEvents();
		} catch (Exception ex) {
			LOGGER.error("Error while starting the correlation: " + ex);
			LogUtils.logException(LOGGER, ex);
		}

	}

	private boolean checkConfigs() {
		boolean success = inputConfig.check();
		success &= correlationConfig.check();
		success &= ontologyConfig.check();
		return success;
	}

	private void processEvents() {
		IInputProcessor inputProcessor = inputProcessorFactory.createInputProcessor();
		Thread processorThread = inputProcessor.process();
		try {
			LOGGER.info("Waiting for input processor to finish.");
			processorThread.join();
			LOGGER.info("Input processor finished.");
		} catch (Exception ex) {
			LOGGER.error("Exception while processing events: " + ex.getMessage());
			LogUtils.logException(LOGGER, ex);
		}
	}
}
