package de.neusta.imonitor.correlation.factory;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.UUID;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.ontology.Ontology;
import de.neusta.imonitor.ontology.constants.NamespaceConstants;

@Service
public class IndividualFactory {

	private static Logger LOGGER = LoggerFactory.getLogger(IndividualFactory.class);

	private static final String IMONITOR_OPERATION_IRI = NamespaceConstants.DEFAULT_NAMESPACE + "Operation";

	@Autowired
	private Ontology ontology;

	/**
	 * Creates a child individual and connects it to the parent individual for a
	 * data object.
	 * 
	 * @param parentIri
	 *            the iri of the parent individual.
	 * @param key
	 *            the key/name of the child to add.
	 * @return the name of the created child as iri.
	 */
	public String createChild(String parentIri, String key) {
		this.ontology.writeLock();
		try {
			String individualIri = createUniqueIndividualName();
			ontology.createIndividual(individualIri, IMONITOR_OPERATION_IRI);
			ontology.addDataProperty(individualIri, NamespaceConstants.DEFAULT_NAMESPACE + "hasName", key);
			ontology.createRelation(individualIri, parentIri, NamespaceConstants.DEFAULT_NAMESPACE + "childOf");
			return individualIri;
		} finally {
			this.ontology.releaseLock();
		}
	}

	/**
	 * Creates the root individual for the data individuals.
	 * 
	 * @param dataGuid
	 *            the guid of the data variable used for naming the individual.
	 * @return the created individual.
	 */
	public String createDataRootIndividual(String dataGuid) {
		try {
			this.ontology.writeLock();
			String individualIri = NamespaceConstants.DEFAULT_NAMESPACE + dataGuid;
			LOGGER.trace("Creating data root node as individual: " + individualIri);
			ontology.createIndividual(individualIri, IMONITOR_OPERATION_IRI);
			return individualIri;
		} finally {
			this.ontology.releaseLock();
		}
	}

	/**
	 * Creates a value of the data object as individual with relations.
	 * 
	 * @param parentIri
	 *            the parent individual iri.
	 * @param key
	 *            the key / variable name for creating an individual.
	 * @param value
	 *            the value / variable value for creating an individual.
	 */
	public String createValueIndividual(String parentIri, String key, String value) {
		try {
			this.ontology.writeLock();
			String individualIri = createUniqueIndividualName();
			ontology.createIndividual(individualIri, IMONITOR_OPERATION_IRI);
			ontology.createRelation(individualIri, parentIri, NamespaceConstants.DEFAULT_NAMESPACE + "childOf");
			ontology.addDataProperty(individualIri, NamespaceConstants.DEFAULT_NAMESPACE + "hasValue", value);
			ontology.addDataProperty(individualIri, NamespaceConstants.DEFAULT_NAMESPACE + "hasName", key);
			return individualIri;
		} finally {
			this.ontology.releaseLock();
		}
	}

	private String createUniqueIndividualName() {
		return NamespaceConstants.DEFAULT_NAMESPACE + UUID.randomUUID().toString();
	}

}
