package de.neusta.imonitor.correlation.garbagecollection;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.neusta.imonitor.correlation.configs.CorrelationConfig;
import de.neusta.imonitor.ontology.Ontology;

/**
 * Garbage collector for individuals.
 *
 */
@Component
public class IndividualGarbageCollector implements Runnable {


	private static final Logger LOGGER = LoggerFactory.getLogger(IndividualGarbageCollector.class);
	private static Logger PERFORMANCE_LOGGER = LoggerFactory.getLogger("performanceLogger");

	@Autowired
	private Ontology ontology;
	
	@Autowired
	private CorrelationConfig correlationConfig;

	private volatile List<String> garbage = new ArrayList<String>();

	private volatile int runningCorrelations;
	
	private int maximum_pool_size = 10000;
	
	@PostConstruct
	public void init() {
		maximum_pool_size = correlationConfig.getIndividualGarbageCollectorSize();
		LOGGER.info("Setting individual garbage collector pool size to: " + maximum_pool_size);
		PERFORMANCE_LOGGER.info("Setting individual garbage collector pool size to: " + maximum_pool_size);
	}

	/**
	 * Adds the given individual iri to the list of individuals to delete.
	 * 
	 * @param iri
	 *            the iri of the individual to delete.
	 */
	public void deleteIndividual(String iri) {
		if (clearingRequired()) {
			LOGGER.warn("Individual is required to be deleted by the garbage collector. Maximum individual garbage pool size reached.");
			removeFirstIndividual();
		}
		garbage.add(iri);
	}

	/**
	 * Called by the correlation, indicating that the correlation is currently
	 * running.
	 */
	public void correlationRunning() {
		runningCorrelations++;
	}

	/**
	 * Called by the correlation, indicating that the correlation has finished.
	 */
	public void correlationFinished() {
		runningCorrelations--;
	}

	@Override
	public void run() {
		LOGGER.trace("Starting individual garbage collector.");
		try {
			while (true) {
				if (isIdle()) {
					removeFirstIndividual();
				}
				Thread.sleep(1);
			}
		} catch (Exception ex) {
			LOGGER.error("Unexpected exception in individual garbage collector.", ex);
		}
	}

	private synchronized void removeFirstIndividual() {
		if (!garbage.isEmpty()) {
			String garbageIndividual = garbage.get(0);
			LOGGER.trace("Garbage collecting individual. Running correlations: " + runningCorrelations);
			try
			{
				ontology.writeLock();
				ontology.deleteIndividual(garbageIndividual);
			} finally {
				ontology.releaseLock();
			}
			garbage.remove(0);
			LOGGER.trace("Garbage entry removed, remaining entries: " + garbage.size());
		}
	}

	private boolean clearingRequired() {
		return garbage.size() > maximum_pool_size;
	}

	private boolean isIdle() {
		return runningCorrelations == 0;
	}

}
