package de.neusta.imonitor.correlation.processors;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.HashMap;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.correlation.services.CommandService;
import de.neusta.imonitor.correlation.services.IncidentService;
import de.neusta.imonitor.ontology.VariableList;
import de.neusta.imonitor.ontology.exception.VariableAlreadyExistsException;
import de.neusta.imonitor.ruleparser.abstraction.Abstraction;
import de.neusta.imonitor.ruleparser.rules.Action;
import de.neusta.imonitor.ruleparser.rules.Rule;

@Service
public class ActionProcessor {

	private static final Logger LOGGER = LoggerFactory.getLogger(ActionProcessor.class);

	@Autowired
	private ExplanationProcessor explanationProcessor;
	@Autowired
	private QueryProcessor queryProcessor;
	@Autowired
	private IncidentService incidentService;
	@Autowired
	private UpdateProcessor updateProcessor;
	@Autowired
	private CommandService commandService;
	@Autowired
	private RecommendationProcessor recommendationProcessor;

	public void processAction(Rule rule, VariableList variableList, Abstraction abstraction) throws VariableAlreadyExistsException  {
		Action action = rule.getAction();
		LOGGER.info("=== Performing action ===");
		addVariablesFromIncident(rule, variableList);
		HashMap<String, List<String>> actionQueryVariables = this.queryProcessor.processQueries(action, variableList, abstraction);
		List<String> explanations = processExplanations(action, variableList, abstraction, actionQueryVariables);
		List<String> recommendations = processRecommendations(action, variableList, actionQueryVariables);
		incidentService.writeIncidentToDatabase(action, variableList, explanations, recommendations);
		commandService.processCommand(action.getCommand(), explanations, recommendations, variableList);
		try {
			this.updateProcessor.process(action, variableList, actionQueryVariables, abstraction);
		} catch (Exception e) {
			LOGGER.error("Could not update ontology.", e);
		}
	}

	private void addVariablesFromIncident(Rule rule, VariableList variableList) throws VariableAlreadyExistsException {
		int risk = rule.getAction().getRisk();
		variableList.createVariable("risk", String.valueOf(risk));
		String incidentName = rule.getAction().getIncidentName();
		variableList.createVariable("incidentname", incidentName);
	}

	private List<String> processRecommendations(Action action, VariableList variableList, HashMap<String, List<String>> actionQueryVariables) {
		List<String> recommendations = recommendationProcessor.processRecommendations(action, variableList, actionQueryVariables);
		for (String recommendation : recommendations) {
			LOGGER.info("Recommendation found: " + recommendation);
		}
		return recommendations;
	}

	private List<String> processExplanations(Action action, VariableList variableList, Abstraction abstraction,
			HashMap<String, List<String>> actionQueryVariables) {
		List<String> explanations = this.explanationProcessor.processExplanations(action, variableList, actionQueryVariables, abstraction);
		for (String explanation : explanations) {
			LOGGER.info("Explanation found: " + explanation);
		}
		return explanations;
	}

}
