package de.neusta.imonitor.correlation.processors;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import de.neusta.imonitor.correlation.garbagecollection.IndividualGarbageCollector;
import de.neusta.imonitor.correlation.model.VariableNode;
import de.neusta.imonitor.correlation.patternmatcher.Pattern;
import de.neusta.imonitor.correlation.patternmatcher.PatternMatcher;
import de.neusta.imonitor.correlation.ruleloader.RuleLoader;
import de.neusta.imonitor.correlation.service.OntologyVariableService;
import de.neusta.imonitor.ontology.Ontology;
import de.neusta.imonitor.ontology.VariableList;
import de.neusta.imonitor.ruleparser.abstraction.Abstraction;
import de.neusta.imonitor.ruleparser.rules.Condition;
import de.neusta.imonitor.ruleparser.rules.Rule;
import de.neusta.imonitor.ruleparser.rules.RuleParser;

@Component
@Scope("prototype")
public class CorrelationEventProcessorTask implements Runnable {

	private static Logger LOGGER = LoggerFactory.getLogger(CorrelationEventProcessorTask.class);
	
	private static Logger PERFORMANCE_LOGGER = LoggerFactory.getLogger("performanceLogger");

	private static final int MAX_ABSTRACTION_LEVEL = 5;

	@Autowired
	private RuleParser ruleParser;

	@Autowired
	private PatternMatcher patternMatcher;
	@Autowired
	private RuleLoader ruleLoader;
	@Autowired
	private Ontology ontology;
	@Autowired
	private ActionProcessor actionProcessor;
	@Autowired
	private OntologyVariableService ontologyVariableService;
	@Autowired
	private IndividualGarbageCollector individualGarbageCollector;

	private VariableList variableList;

	private VariableNode variableGraph;

	@Override
	public void run() {
		long start = System.currentTimeMillis();
		List<String> temporaryVariableIndividuals = new ArrayList<String>();
		try {
			individualGarbageCollector.correlationRunning();
			long startAdd = System.currentTimeMillis();
			temporaryVariableIndividuals = ontologyVariableService.addToOntology(variableGraph, variableList.getVariable("DATA").getValue());
			long finishAdd = System.currentTimeMillis();
			PERFORMANCE_LOGGER.info("add temporary variable individuals took: " + (finishAdd - startAdd) + " ms.");
			LOGGER.trace("Processing event with the following variables:");
			for (de.neusta.imonitor.ontology.Variable variable : variableList.getVariables()) {
				LOGGER.trace("'" + variable.getName() + "': '" + variable.getValue() + "'");
			}
			List<Rule> rules = this.ruleParser.getRules();
			// check rules with increasing abstractions
			for (int level = 0; level < MAX_ABSTRACTION_LEVEL; level++) {
				boolean patternMatchesInThisLevel = false;
				LOGGER.trace("Checking " + rules.size() + " rules with abstraction level: " + level);
				for (Rule rule : rules) {
					Condition condition = rule.getCondition();
					Abstraction abstraction = this.getAbstraction(condition.getRootAbstraction(), level, 0);
					if (abstraction != null) {
						Pattern pattern = new Pattern(abstraction.getQuery());
						boolean patternMatches = false;
						try {
							patternMatches = this.patternMatcher.matchPattern(pattern, variableList);
						} catch (Exception e) {
							LOGGER.error("Exception during processing event.", e);
						}
						if (patternMatches) {
							LOGGER.info("Found matching pattern. Performing actions.");
							this.actionProcessor.processAction(rule, variableList, abstraction);
							patternMatchesInThisLevel = true;
						} else {
							LOGGER.trace("Pattern does not match.");
						}
					} else if (level == 0) {
						LOGGER.error("Could not find abstraction on abstraction level 0 for rule with condition: " + condition.getQuery());
					} else {
						LOGGER.trace("Could not find abstraction on level " + level + " for rule with condition: " + condition.getQuery());
					}
				}
				if (patternMatchesInThisLevel) {
					LOGGER.trace("Aborting matching process since at least one matching rule has been found.");
					return;
				}
			}
		} catch (Exception ex) {
			LOGGER.error("Unexpected exception in event correlation thread.", ex);
		} finally {
			try {
				ontologyVariableService.removeFromOntology(temporaryVariableIndividuals);
			} catch(Exception ex) {
				LOGGER.error("Unexpected exception while removing temporary individuals from ontology.", ex);
			}
			long finish = System.currentTimeMillis();
			PERFORMANCE_LOGGER.info("Correlation took " + (finish-start) + " ms.");
			individualGarbageCollector.correlationFinished();
		}
	}

	private Abstraction getAbstraction(Abstraction abstraction, int maxLevel, int currentLevel) {
		if (abstraction == null) {
			return null;
		}
		if (currentLevel == maxLevel) {
			return abstraction;
		}
		return this.getAbstraction(abstraction.getSuperAbstraction(), maxLevel, currentLevel + 1);
	}

	public void setVariableList(VariableList variableList) {
		this.variableList = variableList;
	}

	public void setVariableGraph(VariableNode variableGraphRoot) {
		this.variableGraph = variableGraphRoot;
		
	}

}
