package de.neusta.imonitor.correlation.processors;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;

import de.neusta.imonitor.correlation.services.QueryFactoryService;
import de.neusta.imonitor.correlation.utils.ConversionUtils;
import de.neusta.imonitor.ontology.Ontology;
import de.neusta.imonitor.ontology.VariableList;
import de.neusta.imonitor.ruleparser.abstraction.Abstraction;
import de.neusta.imonitor.ruleparser.rules.Action;

@Service
public class QueryProcessor {

	private static final Logger LOGGER = LoggerFactory.getLogger(QueryProcessor.class);

	@Autowired
	private Ontology ontology;

	@Autowired
	private QueryFactoryService queryFactoryService;

	/**
	 * processes all query elements, starts a request against the ontology and
	 * returns the variable bindings
	 * 
	 * @param action
	 *            The action that holds the queries.
	 * @param variableList
	 *            A list of variables to replace in the queries.
	 * @param abstraction
	 *            The abstraction from that variable substitutions should be
	 *            used.
	 * @return
	 * @throws Exception
	 */
	public HashMap<String, List<String>> processQueries(Action action, VariableList variableList, Abstraction abstraction) {
		HashMap<String, List<String>> actionQueryVariables = new HashMap<String, List<String>>();
		for (String query : action.getQueries()) {

			this.ontology.readLock();
			try {
				query = queryFactoryService.createQuery(abstraction, query);
				query = ConversionUtils.replaceVariablesInString(variableList, actionQueryVariables, query);
				String queryString = this.ontology.getSparqlPrefixes() + query;
				ResultSet resultSet = this.ontology.select(queryString);
				LOGGER.trace("Performing action select query: '" + queryString + "'");
				// add an empty list of values for each variable in the result
				// set
				List<String> resultVars = resultSet.getResultVars();
				for (String resultVar : resultVars) {
					LOGGER.trace("Adding result variable: '" + resultVar + "'");
					actionQueryVariables.put(resultVar, new ArrayList<String>());
				}
				// add result values
				while (resultSet.hasNext()) {
					QuerySolution solution = resultSet.next();
					for (String variableName : resultVars) {
						RDFNode result = solution.get(variableName);
						List<String> variableEntry = actionQueryVariables.get(variableName);
						Resource resource = result.asResource();
						String value = resource.getLocalName();
						LOGGER.trace("Adding value to action variable list with name '" + variableName + "' and value '" + value + "'");
						variableEntry.add(value);
					}
				}
			} catch (Exception e) {
				LOGGER.error("Exception while handling query.", e);
			} finally {
				this.ontology.releaseLock();
			}
		}
		return actionQueryVariables;
	}

}
