package de.neusta.imonitor.correlation.service;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.correlation.factory.IndividualFactory;
import de.neusta.imonitor.correlation.garbagecollection.IndividualGarbageCollector;
import de.neusta.imonitor.correlation.model.VariableNode;
import de.neusta.imonitor.ontology.Ontology;
import de.neusta.imonitor.ontology.constants.NamespaceConstants;

@Service
public class OntologyVariableService {

	@Autowired
	private IndividualFactory individualFactory;
	@Autowired
	private Ontology ontology;
	@Autowired
	private IndividualGarbageCollector individualGarbageCollector;

	/**
	 * Adds the variable tree to the ontology, rooted with an individual named
	 * by the given dataGuid.
	 * 
	 * @param variableNode
	 *            the variable graph root node.
	 * @param dataGuid
	 *            the dataGuid for referencing the data tree.
	 * @return a list of created individual iris.
	 */
	public List<String> addToOntology(VariableNode variableNode, String dataGuid) {
		List<String> createdIndividuals = new ArrayList<String>();
		createRootNodeInOntology(dataGuid, createdIndividuals);
		internalAddToOntology(variableNode, dataGuid, NamespaceConstants.DEFAULT_NAMESPACE + dataGuid, createdIndividuals);
		return createdIndividuals;
	}

	private void createRootNodeInOntology(String dataGuid, List<String> individuals) {
		String createdIndividual = individualFactory.createDataRootIndividual(dataGuid);
		individuals.add(createdIndividual);
	}

	private void internalAddToOntology(VariableNode variableNode, String dataGuid, String parentIri, List<String> createdIndividuals) {
		addValues(variableNode, dataGuid, parentIri, createdIndividuals);
		addChildren(variableNode, dataGuid, parentIri, createdIndividuals);
	}

	private void addChildren(VariableNode variableNode, String dataGuid, String parentIri, List<String> createdIndividuals) {
		Map<String, VariableNode> children = variableNode.getChildren();
		Set<Entry<String, VariableNode>> childrenEntrySet = children.entrySet();
		for (Entry<String, VariableNode> child : childrenEntrySet) {
			String createdIndividual = individualFactory.createChild(parentIri, child.getKey());
			createdIndividuals.add(createdIndividual);
			String individualIri = createdIndividual;
			internalAddToOntology(child.getValue(), dataGuid, individualIri, createdIndividuals);
		}
	}

	private void addValues(VariableNode variableNode, String dataGuid, String parentIri, List<String> createdIndividuals) {
		Map<String, String> values = variableNode.getValues();
		Set<Entry<String, String>> entrySet = values.entrySet();
		for (Entry<String, String> entry : entrySet) {
			String createdValueIndividual = individualFactory.createValueIndividual(parentIri, entry.getKey(), entry.getValue());
			createdIndividuals.add(createdValueIndividual);
		}
	}


	/**
	 * Removes all the given individuals from the ontology. This can be used to clear the ontology from temporary created individuals.
	 * @param individuals a list of individuals to delete.
	 */
	public void removeFromOntology(List<String> individuals) {
		for (String individual : individuals) {
			individualGarbageCollector.deleteIndividual(individual);
		}
	}
}
