package de.neusta.imonitor.ontology;

/*
 * #%L
 * ontology
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import org.mindswap.pellet.jena.PelletInfGraph;
import org.mindswap.pellet.jena.PelletReasonerFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.clarkparsia.pellet.sparqldl.jena.SparqlDLExecutionFactory;
import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.shared.Lock;
import com.hp.hpl.jena.update.UpdateAction;

import de.neusta.imonitor.ontology.constants.NamespaceConstants;
import de.neusta.imonitor.ontology.utils.LogUtils;

@Scope(value = "singleton")
@Component
public class Ontology {

	private static final Logger LOGGER = LoggerFactory.getLogger(Ontology.class);

	private volatile OntModel model = null;

	/**
	 * Creates a new empty ontology instance.
	 */
	public Ontology() {
		this.createEmptyInstance();
	}

	/**
	 * performs a sparql select query
	 * 
	 * @param queryString
	 * @return
	 */
	public ResultSet select(String queryString) {
		Query query = QueryFactory.create(queryString);
		ResultSet resultSet = null;
		try {
			QueryExecution queryExecution = SparqlDLExecutionFactory.create(query, this.model);
			resultSet = queryExecution.execSelect();
		} catch (Exception ex) {
			LOGGER.error(String.format("Exception while executing the query '%s'.", queryString));
			LogUtils.logException(LOGGER, ex);
			throw ex;
		}
		return resultSet;
	}

	/**
	 * performs a sparql ask query
	 * 
	 * @param queryString
	 * @return
	 * @throws Exception
	 */
	public boolean ask(String queryString) throws Exception {
		try {
			Query query = QueryFactory.create(queryString);
			QueryExecution queryExecution = SparqlDLExecutionFactory.create(query, this.model);
			boolean result = queryExecution.execAsk();
			return result;
		} catch (Exception ex) {
			LOGGER.error(String.format("Exception while executing the query '%s'.", queryString));
			LogUtils.logException(LOGGER, ex);
			throw ex;
		}
	}

	/**
	 * performs a sparql update query
	 * 
	 * @param queryString
	 */
	public void update(String queryString) {
		try {
			UpdateAction.parseExecute(queryString, this.model);
		} catch (Exception ex) {
			LOGGER.error(String.format("Exception while executing the query '%s'.", queryString));
			LogUtils.logException(LOGGER, ex);
			throw ex;
		}
	}

	/**
	 * creates an individual in the ontology
	 * 
	 * @param individualIri
	 *            , the iri of the individual to create.
	 * @param parentClassIri
	 *            , the iri of the class to which the individual should belong
	 * @return the created individual
	 */
	public Individual createIndividual(String individualIri, String parentClassIri) {
		String remappedParentClassIri = this.remapNamespaces(parentClassIri);
		OntClass parentClass = this.model.getOntClass(remappedParentClassIri);
		if (parentClass == null) {
			LOGGER.warn(String.format("Could not find parent class '%s' in the ontology. Trying to create.", remappedParentClassIri));
			parentClass = this.model.createClass(parentClassIri);
		}
		if (parentClass == null) {
			LOGGER.error(String.format("Could not create parent class '%s' in the ontology. Cannot create individual '%s'.", remappedParentClassIri,
					individualIri));
			return null;
		} else {
			String remappedIndividualIri = this.remapNamespaces(individualIri);
			LOGGER.trace("Creating individual: '" + remappedIndividualIri + "'");
			return this.model.createIndividual(remappedIndividualIri, parentClass);
		}
	}

	/**
	 * Creates a relation between two individuals.
	 * 
	 * @param sourceIndividualIri
	 *            the iri of the source individual.
	 * @param targetIndividualIri
	 *            the iri of the target individual.
	 * @param relationIri
	 *            the iri of the relation to use.
	 * @return true on success.
	 */
	public boolean createRelation(String sourceIndividualIri, String targetIndividualIri, String relationIri) {
		Individual sourceIndividual = getIndividual(sourceIndividualIri);
		if (sourceIndividual == null) {
			LOGGER.error("Cannot find source individual '" + sourceIndividualIri + "'.");
			return false;
		}
		Individual targetIndividual = getIndividual(targetIndividualIri);
		if (targetIndividual == null) {
			LOGGER.error("Cannot find target individual '" + targetIndividualIri + "'.");
			return false;
		}
		Property hasChildObjectProperty = getProperty(relationIri);
		if (hasChildObjectProperty == null) {
			LOGGER.error("Cannot find object property '" + relationIri + "'.");
			return false;
		}
		sourceIndividual.addProperty(hasChildObjectProperty, targetIndividual);
		return true;
	}

	/**
	 * returns a list of common prefixes for SPARQL as SPARQL preambel.
	 */
	public String getSparqlPrefixes() {
		return "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>" + "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>"
				+ "PREFIX owl: <http://www.w3.org/2002/07/owl#>" + "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>" + "PREFIX "
				+ NamespaceConstants.DEFAULT_NAMESPACE + " <" + NamespaceConstants.DEFAULT_NAMESPACE_URL + ">\n";
	}

	/**
	 * replaces namespace strings like 'imonitor:' to get an iri
	 * 
	 * @param inputString
	 *            the string for replacing namespaces.
	 * @return the string with replaced namespaces.
	 */
	public String remapNamespaces(String inputString) {
		String result = inputString;
		if (inputString.startsWith(NamespaceConstants.DEFAULT_NAMESPACE)) {
			result = NamespaceConstants.DEFAULT_NAMESPACE_URL + inputString.substring(9);
		}
		return result;
	}

	/**
	 * Gets a concept by the specified url.
	 * 
	 * @param conceptUrl
	 * @return
	 */
	public OntClass getConcept(String conceptUrl) {
		String conceptIri = this.remapNamespaces(conceptUrl);
		OntClass result;
		result = this.model.getOntClass(conceptIri);
		return result;
	}

	/**
	 * Gets an individual by the specified url.
	 * 
	 * @param individualUrl
	 * @return
	 */
	public Individual getIndividual(String individualUrl) {
		String individualIri = this.remapNamespaces(individualUrl);
		Individual result = null;
		result = this.model.getIndividual(individualIri);
		return result;
	}

	/**
	 * Saves the ontology to a file.
	 * 
	 * @param fileName
	 * @return true, on success
	 */
	public boolean save(String fileName) {
		boolean success = true;
		try {
			File file = new File(fileName);
			model.write(new FileOutputStream(file));
			LOGGER.trace(String.format("Ontology saved to '%s'.", fileName));
		} catch (IOException e) {
			LOGGER.error("Exception while saving ontology." + e.getMessage());
			success = false;
		}
		return success;
	}

	/**
	 * Loads an ontology from a file given as parameter.
	 * 
	 * @param fileName
	 *            as uri, e.g. file:data/ontology.owl
	 */
	public void load(String fileName) {
		LOGGER.trace("Loading ontology from file:" + fileName);
		model = ModelFactory.createOntologyModel(PelletReasonerFactory.THE_SPEC);
		model.read(fileName);
		realize();
	}

	/**
	 * Loads the ontology from the given input stream. You cannot save
	 * ontologies loaded this way. Use the load(String filename) method instead.
	 * 
	 * @param inputStream
	 *            the input stream used for loading the ontology.
	 */
	public void load(InputStream inputStream) {
		LOGGER.trace("Loading ontology from stream.");
		model = ModelFactory.createOntologyModel(PelletReasonerFactory.THE_SPEC);
		model.read(inputStream, ""); // incorrect empty url, so you cannot save
										// the ontology.
		realize();
	}

	private void realize() {
		try {
			LOGGER.trace("Realizing and classifying ontology.");
			Graph graph = model.getGraph();
			PelletInfGraph pellet = (PelletInfGraph) graph;
			if (!pellet.isConsistent()) {
				LOGGER.warn("Pellet indicates that the ontology is not consistent.");
			}
			pellet.classify();
			pellet.realize();
		} catch (Exception ex) {
			LOGGER.error("Error while realizing ontology.");
		}
	}

	private void createEmptyInstance() {
		model = ModelFactory.createOntologyModel(PelletReasonerFactory.THE_SPEC);
		realize();
	}

	/**
	 * Gets an property from the ontology. NOTE: Does not perform locking!
	 * 
	 * @param propertyIri
	 *            the iri of the property.
	 * @return the property if found, null otherwise.
	 */
	public Property getProperty(String propertyIri) {
		return this.model.getProperty(this.remapNamespaces(propertyIri));
	}

	/**
	 * Adds a data property to the given individual.
	 * 
	 * @param individualIri
	 *            the iri to the individual.
	 * @param dataTypePropertyIri
	 *            the iri of the data type property.
	 * @param value
	 *            the value to set.
	 * @return true on success.
	 */
	public boolean addDataProperty(String individualIri, String dataTypePropertyIri, String value) {
		Individual individual = getIndividual(individualIri);
		Property property = getProperty(dataTypePropertyIri);
		if (individual == null) {
			LOGGER.error("Could not find individual with iri '" + individualIri + "'.");
			return false;
		}
		if (property == null) {
			LOGGER.error("Could not find data property with iri '" + individualIri + "'.");
			return false;
		}
		individual.addProperty(property, value);
		return true;
	}

	/**
	 * Gets a read lock.
	 */
	public void readLock() {
		this.model.enterCriticalSection(Lock.READ);
	}

	/**
	 * Gets a write lock.
	 */
	public void writeLock() {
		this.model.enterCriticalSection(Lock.WRITE);
	}

	/**
	 * Releases a lock.
	 */
	public void releaseLock() {
		this.model.leaveCriticalSection();
	}

	/**
	 * Deletes the given individual from the ontology.
	 * 
	 * @param individual
	 */
	public void deleteIndividual(String individualIri) {
		LOGGER.trace("Removing individual: '" + individualIri + "'");
		String remappedIndividualIri = this.remapNamespaces(individualIri);
		Individual individual = this.model.getIndividual(remappedIndividualIri);
		individual.remove();
	}
}
