package de.neusta.imonitor.correlation;

import junit.framework.Assert;

import org.junit.Test;

import de.neusta.imonitor.correlation.abstraction.Abstraction;
import de.neusta.imonitor.correlation.abstraction.QueryTranslator;
import de.neusta.imonitor.ontology.Ontology;

public class QueryTranslatorTests {

	@Test
	public void testQueryTranslatorShouldNotTranslateQueryWithoutAbstraction(){
		String testQuery = "ASK { $hoststate owl:sameAs imonitor:DOWN. ?process a imonitor:Process. ?process imonitor:dependsOn $hostaddress. ?process imonitor:hasCriticality imonitor:HIGH_CRITICALITY. }";
		Ontology ontology = TestHelper.getTestOntology();
		QueryTranslator queryTranslator = new QueryTranslator();
		try {
			Abstraction abstractions = queryTranslator.buildAbstractions(testQuery, ontology);
			Assert.assertEquals(0, abstractions.getAbstractions().length);
			Assert.assertEquals(testQuery, abstractions.getQuery());
		} catch (Exception e) {
			e.printStackTrace();
			Assert.fail();
		}
	}
	
	@Test
	public void testQueryTranslatorShouldReplaceTheAbstractionFunction(){
		String testQuery = "ASK { $hoststate owl:sameAs Abstract(imonitor:WebServer, imonitor:WebServer, \"testVar\") }";
		Ontology ontology = TestHelper.getTestOntology();
		QueryTranslator queryTranslator = new QueryTranslator();
		try {
			Abstraction abstraction = queryTranslator.buildAbstractions(testQuery, ontology);
			Assert.assertEquals("ASK { $hoststate owl:sameAs <http://www.imonitor-project.com/ontology#WebServer> }", abstraction.getQuery());
			Assert.assertEquals(1, abstraction.getAbstractionVariables().length);
			Assert.assertEquals("testVar", abstraction.getAbstractionVariables()[0].getVariableName());
			Assert.assertEquals("<http://www.imonitor-project.com/ontology#WebServer>", abstraction.getAbstractionVariables()[0].getVariableValue());
		} catch (Exception e) {
			e.printStackTrace();
			Assert.fail();
		}
	}
	
	@Test
	public void testQueryTranslatorShouldReplaceTheAbstractionFunctionAndBuildAbstraction(){
		String testQuery = "ASK { $hoststate owl:sameAs Abstract(imonitor:WebServer, imonitor:Server, \"testVar\") }";
		Ontology ontology = TestHelper.getTestOntology();
		QueryTranslator queryTranslator = new QueryTranslator();
		try {
			Abstraction abstractions = queryTranslator.buildAbstractions(testQuery, ontology);
			Assert.assertEquals("ASK { $hoststate owl:sameAs <http://www.imonitor-project.com/ontology#WebServer> }", abstractions.getQuery());
			Assert.assertEquals("ASK { $hoststate owl:sameAs <http://www.imonitor-project.com/ontology#Server> }", abstractions.getSuperAbstraction().getQuery());
			Assert.assertEquals(0, abstractions.getAbstractions()[0]);
			Assert.assertEquals(1, abstractions.getSuperAbstraction().getAbstractions()[0]);
		} catch (Exception e) {
			e.printStackTrace();
			Assert.fail();
		}
	} 
	
	@Test
	public void testQueryTranslatorShouldReplaceTheAbstractionFunctionAndLimitTheAbstractions(){
		String testQuery = "ASK { $hoststate owl:sameAs Abstract(imonitor:WebServer, imonitor:WebServer, \"testVar\") }";
		Ontology ontology = TestHelper.getTestOntology();
		QueryTranslator queryTranslator = new QueryTranslator();
		try {
			Abstraction abstractions = queryTranslator.buildAbstractions(testQuery, ontology);
			Assert.assertEquals("ASK { $hoststate owl:sameAs <http://www.imonitor-project.com/ontology#WebServer> }", abstractions.getQuery());
			Assert.assertNull(abstractions.getSuperAbstraction());
		} catch (Exception e) {
			e.printStackTrace();
			Assert.fail();
		}
	}
	
	@Test
	public void testQueryTranslatorShouldReplaceMultipleAbstractionFunctions(){
		String testQuery = "ASK { $hoststate owl:sameAs Abstract(imonitor:WebServer, imonitor:WebServer, \"testVar\"). $hoststate owl:sameAs Abstract(imonitor:WebServer, imonitor:WebServer, \"testVar2\") }";
		Ontology ontology = TestHelper.getTestOntology();
		QueryTranslator queryTranslator = new QueryTranslator();
		try {
			Abstraction abstractions = queryTranslator.buildAbstractions(testQuery, ontology);
			Assert.assertEquals("ASK { $hoststate owl:sameAs <http://www.imonitor-project.com/ontology#WebServer>. $hoststate owl:sameAs <http://www.imonitor-project.com/ontology#WebServer> }", abstractions.getQuery());
			Assert.assertNull(abstractions.getSuperAbstraction());
		} catch (Exception e) {
			e.printStackTrace();
			Assert.fail();
		}
	} 

	
	@Test
	public void testQueryTranslatorShouldReplaceMultipleAbstractionFunctionsAndBuildAbstractions(){
		String testQuery = "ASK { $hoststate owl:sameAs Abstract(imonitor:WebServer, imonitor:Server, \"testVar\"). $hoststate owl:sameAs Abstract(imonitor:WebServer, imonitor:Server, \"testVar2\") }";
		Ontology ontology = TestHelper.getTestOntology();
		QueryTranslator queryTranslator = new QueryTranslator();
		try {
			Abstraction abstractions = queryTranslator.buildAbstractions(testQuery, ontology);
			Assert.assertEquals("ASK { $hoststate owl:sameAs <http://www.imonitor-project.com/ontology#WebServer>. $hoststate owl:sameAs <http://www.imonitor-project.com/ontology#WebServer> }", abstractions.getQuery());
			Assert.assertNotNull(abstractions.getSuperAbstraction());
			Assert.assertEquals("ASK { $hoststate owl:sameAs <http://www.imonitor-project.com/ontology#WebServer>. $hoststate owl:sameAs <http://www.imonitor-project.com/ontology#Server> }", abstractions.getSuperAbstraction().getQuery());
			Assert.assertNotNull(abstractions.getSuperAbstraction().getSuperAbstraction());
			Assert.assertEquals("ASK { $hoststate owl:sameAs <http://www.imonitor-project.com/ontology#Server>. $hoststate owl:sameAs <http://www.imonitor-project.com/ontology#Server> }", abstractions.getSuperAbstraction().getSuperAbstraction().getQuery());
			Assert.assertEquals(0, abstractions.getAbstractions()[0]);
			Assert.assertEquals(0, abstractions.getAbstractions()[1]);
			Assert.assertEquals(0, abstractions.getSuperAbstraction().getAbstractions()[0]);
			Assert.assertEquals(1, abstractions.getSuperAbstraction().getAbstractions()[1]);
			Assert.assertEquals(1, abstractions.getSuperAbstraction().getSuperAbstraction().getAbstractions()[0]);
			Assert.assertEquals(1, abstractions.getSuperAbstraction().getSuperAbstraction().getAbstractions()[1]);
		} catch (Exception e) {
			e.printStackTrace();
			Assert.fail();
		}
	} 

	
}
