package de.neusta.imonitor.ruleparser.abstraction;

/*
 * #%L
 * ruleparser
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

import de.neusta.imonitor.ontology.Ontology;

@Service
public class QueryToAbstractionTransformer {

	private static Logger LOGGER = LoggerFactory.getLogger(QueryToAbstractionTransformer.class);

	@Autowired
	private Ontology ontology;

	/**
	 * analyzes a SPARQL query and finds abstractions in the statement it
	 * replaces the abstractions in the query by the most specific concept
	 * 
	 * @param query
	 * @return returns all abstractions
	 * @throws Exception
	 */
	public Abstraction transform(String query) throws Exception {
		List<AbstractionFunctionOccurence> abstractionFunctions = scanForAbstractionFunctions(query);

		AbstractionVariable[] abstractionVariables = new AbstractionVariable[abstractionFunctions.size()];
		for (int i = 0; i < abstractionFunctions.size(); i++) {
			abstractionVariables[i] = new AbstractionVariable();
			abstractionVariables[i].setVariableName(abstractionFunctions.get(i).variableName);
		}
		Abstraction abstraction = buildAbstractions(query, abstractionFunctions, null, new int[abstractionFunctions.size()], abstractionVariables);

		return abstraction;
	}

	private Abstraction buildAbstractions(String query, List<AbstractionFunctionOccurence> abstractionFunctions, Abstraction parent, int[] abstractions,
			AbstractionVariable[] abstractionVariables) {

		Abstraction abstraction = new Abstraction();
		abstraction.setAbstractions(abstractions);
		AbstractionVariable[] newAbstractionVariables = new AbstractionVariable[abstractionVariables.length];
		for (int i = 0; i < abstractionVariables.length; i++) {
			newAbstractionVariables[i] = abstractionVariables[i].clone();
		}
		String abstractionQuery = this.buildQueryForAbstraction(abstraction, query, abstractionFunctions, newAbstractionVariables);
		abstraction.setAbstractionVariables(newAbstractionVariables);
		// check if an abstraction exists
		boolean abstractionExists = abstractionQuery != null;
		if (!abstractionExists) {
			return null;
		}
		// set values of the new abstraction
		abstraction.setSubAbstraction(parent);
		if (parent != null) {
			parent.setSuperAbstraction(abstraction);
		}

		abstraction.setQuery(abstractionQuery);
		// build abstractions
		for (int abstractId = 0; abstractId < abstractions.length; abstractId++) {
			int[] newAbstractions = abstractions.clone();
			newAbstractions[abstractId]++;
			this.buildAbstractions(query, abstractionFunctions, abstraction, newAbstractions, newAbstractionVariables);
		}
		return abstraction;
	}

	private String buildQueryForAbstraction(Abstraction abstraction, String query, List<AbstractionFunctionOccurence> abstractionFunctions,
			AbstractionVariable[] abstractionVariables) {
		String result = query;
		int relativeIndex = 0;
		// replace all abstraction functions
		for (int i = 0; i < abstractionFunctions.size(); i++) {
			AbstractionFunctionOccurence abstractionFunction = abstractionFunctions.get(i);
			String abstractedConcept = getAbstractedConcept(abstractionFunction.concept, abstractionFunction.mostAbstractConcept,
					abstraction.getAbstractions()[i]);
			abstractionVariables[i].setVariableName(abstractionFunction.variableName);
			abstractionVariables[i].setVariableValue(abstractedConcept);
			if (abstractedConcept == null) {
				return null;
			}
			result = result.substring(0, abstractionFunction.abstractionFunctionStartIndex + relativeIndex) + abstractedConcept
					+ result.substring(abstractionFunction.abstractionFunctionEndIndex + 1 + relativeIndex);
			int abstractionFunctionLength = abstractionFunction.abstractionFunctionEndIndex - abstractionFunction.abstractionFunctionStartIndex + 1; // +1
																																						// for
																																						// the
																																						// final
																																						// bracket
			relativeIndex += abstractedConcept.length() - abstractionFunctionLength;
		}
		return result;
	}

	private String getAbstractedConcept(String concept, String mostAbstractConcept, int abstractions) {
		LOGGER.info("Fetching concept '" + concept + "'.");
		this.ontology.readLock();
		try {
			OntClass targetClass = this.ontology.getConcept(concept);
			if (targetClass == null) {
				LOGGER.warn(String.format("Could not find concept to abstract: '%s'", concept));
				return null;
			}
			for (int abstraction = 0; abstraction < abstractions; abstraction++) {
				// too many abstractions requested, return null
				if (targetClass == null) {
					return null;
				}
				// cannot abstract any more, most abstract concept already
				// reached
				if (targetClass.getURI().equals(ontology.remapNamespaces(mostAbstractConcept))) {
					return null;
				}
				ExtendedIterator<OntClass> directSuperClasses = targetClass.listSuperClasses(true);
				if (directSuperClasses.hasNext()) {
					targetClass = directSuperClasses.next();
				} else {
					targetClass = null;
				}
			}
			if (targetClass == null) {
				return null;
			}
			return "<" + targetClass.getURI() + ">";
		} finally {
			this.ontology.releaseLock();
		}
	}

	private List<AbstractionFunctionOccurence> scanForAbstractionFunctions(String query) throws Exception {
		int abstractionFunctionStartIndex;
		List<AbstractionFunctionOccurence> abstractionFunctionOccurences = new ArrayList<AbstractionFunctionOccurence>();
		int currentIndex = 0;
		do {
			// scan for abstraction functions
			String lowerCaseQuery = query.toLowerCase();
			abstractionFunctionStartIndex = lowerCaseQuery.indexOf("abstract(", currentIndex);
			if (abstractionFunctionStartIndex >= 0) {
				int abstractionFunctionEndIndex = lowerCaseQuery.indexOf(")", abstractionFunctionStartIndex);
				String argumentsString = query.substring(abstractionFunctionStartIndex + 9, abstractionFunctionEndIndex);
				StringTokenizer tokenizer = new StringTokenizer(argumentsString, ",");
				if (tokenizer.countTokens() != 3) {
					throw new Exception("Invalid number of parameters for the abstract function in the rule.");
				}
				String concept = tokenizer.nextToken();
				String mostAbstractConcept = tokenizer.nextToken();
				String variableName = tokenizer.nextToken();

				AbstractionFunctionOccurence abstractionFunctionOccurence = new AbstractionFunctionOccurence();
				abstractionFunctionOccurence.abstractionFunctionStartIndex = abstractionFunctionStartIndex;
				abstractionFunctionOccurence.abstractionFunctionEndIndex = abstractionFunctionEndIndex;
				abstractionFunctionOccurence.concept = QueryHelper.removeQuotationMarks(concept.trim());
				abstractionFunctionOccurence.mostAbstractConcept = QueryHelper.removeQuotationMarks(mostAbstractConcept.trim());
				abstractionFunctionOccurence.variableName = QueryHelper.removeQuotationMarks(variableName.trim());
				abstractionFunctionOccurences.add(abstractionFunctionOccurence);
				currentIndex = abstractionFunctionEndIndex;
			}
		} while (abstractionFunctionStartIndex >= 0);
		return abstractionFunctionOccurences;
	}

	public class AbstractionFunctionOccurence {
		public int abstractionFunctionStartIndex;
		public int abstractionFunctionEndIndex;
		public String concept;
		public String mostAbstractConcept;
		public String variableName;
	}
}
