package de.neusta.imonitor.ruleparser.rules;

/*
 * #%L
 * ruleparser
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.io.StringReader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.ruleparser.abstraction.Abstraction;
import de.neusta.imonitor.ruleparser.abstraction.QueryToAbstractionTransformer;
import de.neusta.imonitor.ruleparser.exception.RuleValidationException;
import de.neusta.imonitor.ruleparser.validator.RuleValidator;

/**
 * Parses a rule or a set of rules and adds it to a local list of rules.
 *
 */
@Service
public class RuleParser {

	private List<Rule> rules = new ArrayList<Rule>();

	private static final Logger LOGGER = LoggerFactory.getLogger(RuleParser.class);

	@Autowired
	private QueryToAbstractionTransformer queryTranslator;
	@Autowired
	private RuleValidator ruleValidator;
	
	/**
	 * Parses a given rules xml document.
	 * 
	 * @param document
	 *            The document to parse.
	 * @return Returns true on success.
	 * @throws RuleValidationException 
	 */
	@SuppressWarnings("rawtypes")
	public boolean parseDocument(Document document) throws RuleValidationException {
		this.rules = new ArrayList<Rule>();
		Element root = document.getRootElement();
		if (!root.getName().toLowerCase().equals("rules")) {
			LOGGER.error(String.format("Invalid root element name. Expected 'rules' but was '%s'.", root.getName()));
			return false;
		}
		for (Iterator i = root.elementIterator(); i.hasNext();) {
			Element element = (Element) i.next();
			if (!parseRuleElement(element)) {
				return false;
			}
		}
		return true;
	}

	private boolean parseRuleElement(Element element) throws RuleValidationException {
		String elementNameLowerCase = element.getName().toLowerCase();
		if (elementNameLowerCase.equals("rule")) {
			Rule rule;
			try {
				rule = this.parseRule(element);
			} catch (Exception e) {
				LOGGER.error("Exception while parsing rule.", e);
				return false;
			}
			ruleValidator.validate(rule);
			this.rules.add(rule);
		} else {
			LOGGER.warn(String.format("Unrecognized element in rule file (inside the rule element): '%s'", elementNameLowerCase));
			return false;
		}
		return true;
	}

	/**
	 * Cleares the list of parsed rules.
	 */
	public void clear() {
		this.rules = new ArrayList<Rule>();
	}

	/**
	 * Parses a single rule from xml and adds it to the list of rules hold by
	 * this parser.
	 * 
	 * @param document
	 *            the xml document to parse.
	 * @throws RuleValidationException 
	 */
	public boolean parseAndAddRule(Document document) throws RuleValidationException {
		Element rootElement = document.getRootElement();
		if (!"rule".equalsIgnoreCase(rootElement.getName())) {
			LOGGER.warn("The give rule xml has no rule root element. The root element name is: '" + rootElement.getName() + "'.");
			return false;
		}
		return parseRuleElement(rootElement);
	}

	/**
	 * Parses a single rule xml.
	 * 
	 * @param ruleXml
	 *            The xml for one rule.
	 * @return Returns the rule.
	 * @throws Exception
	 *             Exception during parsing may occur.
	 */
	public Rule parseRule(String ruleXml) throws Exception {
		SAXReader reader = new SAXReader();
		Document document = reader.read(new StringReader(ruleXml));
		Element rootElement = document.getRootElement();
		if (!rootElement.getName().equalsIgnoreCase("rule")) {
			LOGGER.warn(String.format("Unrecognized element in rule file: '%s'", rootElement.getName()));
			return null;
		}
		return this.parseRule(rootElement);
	}

	@SuppressWarnings("rawtypes")
	private Rule parseRule(Element parentElement) throws Exception {
		Rule rule = new Rule();
		for (Iterator i = parentElement.elementIterator(); i.hasNext();) {
			Element element = (Element) i.next();
			String elementNameLowerCase = element.getName().toLowerCase();
			if (elementNameLowerCase.equals("condition")) {
				Condition condition = this.parseCondition(element);
				rule.setCondition(condition);
			} else if (elementNameLowerCase.equals("action")) {
				Action action = this.parseAction(element);
				rule.setAction(action);
			} else {
				LOGGER.warn(String.format("Unrecognized element in rule file: '%s'", elementNameLowerCase));
			}
		}
		return rule;
	}

	@SuppressWarnings("rawtypes")
	private Action parseAction(Element parentElement) {
		Action action = new Action();
		for (Iterator i = parentElement.elementIterator(); i.hasNext();) {
			Element element = (Element) i.next();
			// variable
			String elementNameLowerCase = element.getName().toLowerCase();
			if (elementNameLowerCase.equals("incident")) {
				Incident incident = this.parseIncident(element);
				action.addIncident(incident);
			} else if (elementNameLowerCase.equals("command")) {
				String command = this.parseCommand(element);
				action.setCommand(command);
			} else if (elementNameLowerCase.equals("query")) {
				String query = this.parseActionQuery(element);
				action.addQuery(query);
			} else if (elementNameLowerCase.equals("updatequery")) {
				String query = this.parseUpdateQuery(element);
				action.addUpdateQuery(query);
			} else if (elementNameLowerCase.equals("recommendation")) {
				String recommendation = this.parseRecommendation(element);
				action.addRecommendation(recommendation);
			} else if (elementNameLowerCase.equals("explanation")) {
				String explanation = this.parseExplanation(element);
				action.addExplanation(explanation);
			} else if (elementNameLowerCase.equals("risk")) {
				int risk = this.parseRisk(element);
				action.setRisk(risk);
			} else if (elementNameLowerCase.equals("incidentname")) {
				String incidentName = this.parseIncidentName(element);
				action.setIncidentName(incidentName);
			} else {
				LOGGER.warn(String.format("Unrecognized element in rule file (inside the action element): '%s'", elementNameLowerCase));
			}
		}
		return action;
	}

	private String parseIncidentName(Element element) {
		String stringValue = element.getStringValue();
		return stringValue;
	}

	private int parseRisk(Element element) {
		String stringValue = element.getStringValue();
		return Integer.parseInt(stringValue);
	}

	private Incident parseIncident(Element element) {
		Incident incident = new Incident();
		incident.setUri(element.attributeValue("iri"));
		incident.setName(element.attributeValue("name"));
		return incident;
	}

	private String parseActionQuery(Element element) {
		String actionQuery = element.getStringValue();
		return actionQuery;
	}

	private String parseUpdateQuery(Element element) {
		String updateQuery = element.getStringValue();
		return updateQuery;
	}

	private String parseRecommendation(Element element) {
		String recommendation = element.getStringValue();
		return recommendation;
	}

	private String parseExplanation(Element element) {
		String explanation = element.getStringValue();
		return explanation;
	}

	private String parseCommand(Element element) {
		String command = element.getStringValue();
		return command;
	}

	@SuppressWarnings("rawtypes")
	private Condition parseCondition(Element parentElement) throws Exception {
		Condition condition = new Condition();
		for (Iterator i = parentElement.elementIterator(); i.hasNext();) {
			Element element = (Element) i.next();
			// variable
			String elementNameLowerCase = element.getName().toLowerCase();
			if (elementNameLowerCase.equals("query")) {
				String query = this.parseQuery(element);
				Abstraction rootAbstraction = queryTranslator.transform(query);
				condition.setQuery(query);
				condition.setRootAbstraction(rootAbstraction);
			} else {
				LOGGER.warn(String.format("Unrecognized element in rule file: '%s'", elementNameLowerCase));
			}
		}
		return condition;
	}

	private String parseQuery(Element element) {
		String query = element.getStringValue();
		return query;
	}

	public List<Rule> getRules() {
		return this.rules;
	}

}
