package de.neusta.imonitor.ruleparser.validator;

/*
 * #%L
 * ruleparser
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.List;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import de.neusta.imonitor.ontology.Ontology;
import de.neusta.imonitor.ruleparser.exception.RuleValidationException;
import de.neusta.imonitor.ruleparser.model.FailureEnum;
import de.neusta.imonitor.ruleparser.rules.Action;
import de.neusta.imonitor.ruleparser.rules.Rule;

@Service
public class RuleValidator {

	private static final String USE_ABSTRACT_FUNCTION_REGEX = "(?i)useabstract\\(.*?\\)";
	private static final String ABSTRACT_FUNCTION_REGEX = "(?i)abstract\\(.*?\\)";
	@Autowired
	private Ontology ontology;

	/**
	 * Validates the given rule.
	 * 
	 * @param rule
	 *            the rule to validate.
	 * @throws RuleValidationException
	 *             exception if the rule is not valid.
	 */
	public void validate(Rule rule) throws RuleValidationException {
		if (rule.getCondition() == null) {
			throw new RuleValidationException(FailureEnum.MISSING_CONDITIONS);
		}
		validateSparqlCondition(rule.getCondition().getQuery());
		Action action = rule.getAction();
		validateAction(action);
	}

	private void validateAction(Action action) throws RuleValidationException {
		if (action == null) {
			throw new RuleValidationException(FailureEnum.MISSING_ACTIONS);
		} else {
			List<String> queries = action.getQueries();
			validateSelectQueries(queries);
			List<String> updateQueries = action.getUpdateQueries();
			validateUpdateQueries(updateQueries);
		}
	}

	private void validateUpdateQueries(List<String> updateQueries) throws RuleValidationException {
		for (String query : updateQueries) {
			validateSparqlUpdate(query);
		}
	}

	private void validateSelectQueries(List<String> queries) throws RuleValidationException {
		for (String query : queries) {
			validateSparqlSelect(query);
		}
	}

	
	private void validateSparqlUpdate(String query) throws RuleValidationException {
		String finalQuery = ontology.getSparqlPrefixes() + replaceFunctionsAndVariables(query);
		try {
			this.ontology.writeLock();
			ontology.update(finalQuery);
		} catch (Exception e) {
			throw new RuleValidationException(FailureEnum.SPARQL_SYNTAX_ERROR, e, finalQuery);
		} finally {
			this.ontology.releaseLock();
		}
	}

	private void validateSparqlSelect(String query) throws RuleValidationException {
		String finalQuery = ontology.getSparqlPrefixes() + replaceFunctionsAndVariables(query);
		try {
			this.ontology.readLock();
			ontology.select(finalQuery);
		} catch (Exception e) {
			throw new RuleValidationException(FailureEnum.SPARQL_SYNTAX_ERROR, e, finalQuery);
		} finally {
			this.ontology.releaseLock();
		}
	}

	private void validateSparqlCondition(String query) throws RuleValidationException {
		String finalQuery = ontology.getSparqlPrefixes() + replaceFunctionsAndVariables(query);
		try {
			this.ontology.readLock();
			ontology.ask(finalQuery);
		} catch (Exception e) {
			throw new RuleValidationException(FailureEnum.SPARQL_SYNTAX_ERROR, e, finalQuery);
		} finally {
			this.ontology.releaseLock();
		}
	}
	
	private String replaceFunctionsAndVariables(String query) {
		if (StringUtils.isEmpty(query)) {
			return query;
		}
		String result = query.replaceAll(ABSTRACT_FUNCTION_REGEX, "?" +UUID.randomUUID().toString());
		result = result.replaceAll(USE_ABSTRACT_FUNCTION_REGEX, "?" +UUID.randomUUID().toString());
		result = result.replaceAll("\\$", "");
		return result;
	}

}
