package de.neusta.imonitor.ruleparser.rules;

/*
 * #%L
 * ruleparser
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import static org.hamcrest.CoreMatchers.is;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;

import java.io.StringReader;
import java.net.URL;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.io.SAXReader;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import de.neusta.imonitor.ruleparser.abstraction.QueryToAbstractionTransformer;
import de.neusta.imonitor.ruleparser.validator.RuleValidator;


@RunWith(MockitoJUnitRunner.class)
public class RuleParserTest {
	
	private static final String TEST_RULES_FILENAME = "rules-test.xml";
	
	@InjectMocks
	private RuleParser ruleParser;
	
	@Mock
	private QueryToAbstractionTransformer queryTranslator;
	@Mock
	private RuleValidator ruleValidator;

	private Document document;
	
	@Before
	public void setup() throws DocumentException {
		SAXReader reader = new SAXReader();
		URL testUrl = Thread.currentThread().getContextClassLoader().getResource(TEST_RULES_FILENAME);
		document = reader.read(testUrl);
	}

	@Test
	public void testParseDocument() throws Exception {
		boolean result = ruleParser.parseDocument(document );
		
		assertTrue(result);
		List<Rule> rules = ruleParser.getRules();
		assertEquals(1, rules.size());
		Mockito.verify(ruleValidator).validate(Mockito.isA(Rule.class));
	}
	
	

	@Test
	public void testParseDocumentExample() throws Exception {
		String xmlData = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><rule><action/><condition/></rule>";
		SAXReader reader = new SAXReader();
		Document document = reader.read(new StringReader(xmlData));
		
		boolean parseDocument = ruleParser.parseAndAddRule(document);
		
		assertThat(parseDocument, is(true));
		assertThat(ruleParser.getRules().size(), is(1));
	}
	
	@Test
	public void testParseDocumentExample2() throws Exception {
		String xmlData = " <rule>  <description>Updates the host state in the operations part of the ontology.</description>  <condition>   <query>ASK {{$hoststate rdf:type/rdfs:subClassOf* imonitor:Down.} UNION { $hoststate rdf:type/rdfs:subClassOf* imonitor:Up.} UNION { $hoststate rdf:type/rdfs:subClassOf* imonitor:Unreachable.}}</query>  </condition>  	<action>		<updatequery>INSERT {imonitor:hostDown$GUID1 a imonitor:HostStateOperation } WHERE {}</updatequery>		<updatequery>INSERT DATA { imonitor:hostDown$GUID1 imonitor:hostaddress $hostaddress. }</updatequery>		<updatequery>INSERT DATA { imonitor:hostDown$GUID1 imonitor:hoststate $hoststate. }</updatequery>	</action> </rule>";
		SAXReader reader = new SAXReader();
		Document document = reader.read(new StringReader(xmlData));
		
		boolean parseDocument = ruleParser.parseAndAddRule(document);
		
		assertThat(parseDocument, is(true));
	}
	

	@Test
	public void testParseUpdateQuery() throws Exception {
		ruleParser.parseDocument(document );
		
		List<Rule> rules = ruleParser.getRules();
		Rule rule = rules.get(0);
		Action action = rule.getAction();
		List<String> updateQueries = action.getUpdateQueries();
		assertEquals(1, updateQueries.size());
		assertEquals("UPDATE {imonitor:myIncident a imonitor:Incident } WHERE {}", updateQueries.get(0));
	}
	
	
	@Test
	public void testParseIncidentName() throws Exception {
		ruleParser.parseDocument(document );
		
		List<Rule> rules = ruleParser.getRules();
		Rule rule = rules.get(0);
		Action action = rule.getAction();
		String incidentName = action.getIncidentName();
		assertEquals("incident_name", incidentName);
	}

	
	@Test
	public void testParseRisk() throws Exception {
		ruleParser.parseDocument(document );
		
		List<Rule> rules = ruleParser.getRules();
		Rule rule = rules.get(0);
		Action action = rule.getAction();
		int risk = action.getRisk();
		assertEquals(10, risk);
	}


	@Test
		public void testParseAndAddRuleString() throws Exception {
			Rule rule = ruleParser.parseRule("<rule></rule>");
			
			assertNotNull(rule);
		}

	

}
