package de.neusta.imonitor.ruleparser.validator;

/*
 * #%L
 * ruleparser
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import org.junit.Before;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import de.neusta.imonitor.ontology.Ontology;
import de.neusta.imonitor.ruleparser.model.FailureEnum;
import de.neusta.imonitor.ruleparser.rules.Action;
import de.neusta.imonitor.ruleparser.rules.Condition;
import de.neusta.imonitor.ruleparser.rules.Rule;


@RunWith(MockitoJUnitRunner.class)
public class RuleValidatorTest {
	
	private static final String PREFIX = "prefix";
	@InjectMocks
	private RuleValidator ruleValidator;
	@Mock
	private Ontology ontology;
	
	@org.junit.Rule
	public ExpectedException exception = ExpectedException.none();
	
	@Before
	public void setup() {
		Mockito.when(ontology.getSparqlPrefixes()).thenReturn(PREFIX);
	}

	@Test
	public void testValidateMissingCondition() throws Exception {
		Rule rule = new Rule();
		exception.expect(new RuleValidationExceptionMatcher(FailureEnum.MISSING_CONDITIONS));
		
		ruleValidator.validate(rule );
	}
	
	@Test
	public void testValidateMissingAction() throws Exception {
		Rule rule = new Rule();
		Condition condition = new Condition();
		rule.setCondition(condition );
		exception.expect(new RuleValidationExceptionMatcher(FailureEnum.MISSING_ACTIONS));
		
		ruleValidator.validate(rule );
	}

	
	@Test
	public void testValidate() throws Exception {
		Rule rule = new Rule();
		Condition condition = new Condition();
		condition.setQuery("ASK {}");
		Action action = new Action();
		rule.setAction(action );
		rule.setCondition(condition );
		
		ruleValidator.validate(rule );
	}

	@Test
	public void testValidateThrowsExceptionInvalidSparqlOnCondition() throws Exception {
		Rule rule = new Rule();
		Condition condition = new Condition();
		condition.setQuery("wrong syntax");
		rule.setCondition(condition );
		exception.expect(new RuleValidationExceptionMatcher(FailureEnum.SPARQL_SYNTAX_ERROR));
		Mockito.when(ontology.ask(PREFIX + "wrong syntax")).thenThrow(new Exception("sparql exception"));
		
		ruleValidator.validate(rule );
	}
	
	@Test
	public void testValidateThrowsExceptionInvalidSparqlOnActionQueries() throws Exception {
		Rule rule = new Rule();
		Condition condition = new Condition();
		condition.setQuery("right syntax");
		rule.setCondition(condition );
		Action action = new Action();
		action.addQuery("wrong syntax");
		rule.setAction(action );
		exception.expect(new RuleValidationExceptionMatcher(FailureEnum.SPARQL_SYNTAX_ERROR));
		Mockito.doThrow(new RuntimeException("sparql exception")).when(ontology).select(PREFIX + "wrong syntax");
		
		ruleValidator.validate(rule );
	}
	
	@Test
	public void testValidateThrowsExceptionInvalidSparqlOnActionUpdateQueries() throws Exception {
		Rule rule = new Rule();
		Condition condition = new Condition();
		condition.setQuery("right syntax");
		rule.setCondition(condition );
		Action action = new Action();
		action.addQuery("right syntax");
		action.addUpdateQuery("wrong syntax");
		rule.setAction(action );
		exception.expect(new RuleValidationExceptionMatcher(FailureEnum.SPARQL_SYNTAX_ERROR));
		Mockito.doThrow(new RuntimeException("sparql exception")).when(ontology).update(PREFIX + "wrong syntax");
		
		ruleValidator.validate(rule );
	}
}
