package de.neusta.imonitor.rulewebservice.services;

/*
 * #%L
 * rulewebservice
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.knowledgeserver.webservice.RuleDTO;
import de.neusta.imonitor.knowledgeserver.webservice.RuleService;
import de.neusta.imonitor.ruleparser.independencychecker.IndependentRuleChecker;
import de.neusta.imonitor.ruleparser.rules.Rule;
import de.neusta.imonitor.ruleparser.rules.RuleParser;
import de.neusta.imonitor.rulewebservice.factory.RuleWebServiceFactory;

/**
 * The rule web service. Please set the url property if you would not use the url specified in the config file. 
 *
 */
@Service
@Scope("singleton")
public class RuleWebService {

	private static final Logger LOGGER = LoggerFactory.getLogger(RuleWebService.class);

	@Autowired
	private RuleWebServiceFactory ruleWebServiceFactory;
	@Autowired
	private IndependentRuleChecker independentRuleChecker;
	@Autowired
	private RuleParser ruleParser;

	private RuleService ruleService;

	private URL url;

	/**
	 * performs lazy initialization.
	 * 
	 * @throws MalformedURLException
	 */
	public void init() throws MalformedURLException {
		if (ruleService == null) {
			if (getUrl() != null) {
				ruleService = ruleWebServiceFactory.create(getUrl());
			} else {
				ruleService = ruleWebServiceFactory.create();
			}
		}
	}

	/**
	 * Fetches all rules form the server.
	 * 
	 * @return The list of all rules.
	 * @throws MalformedURLException
	 */
	public List<RuleDTO> getRules() throws MalformedURLException {
		init();
		return ruleService.getRules();
	}

	/**
	 * Commits a rule to the server.
	 * 
	 * @param rule
	 *            The rule to commit.
	 * @return true on success.
	 * @throws Exception
	 *             Exception if the rule could not be parsed.
	 * @throws NotIndependentException
	 *             if the rule was not independent.
	 */
	public boolean commitRule(RuleDTO rule) throws Exception {
		init();
		Rule parsedRule = ruleParser.parseRule(rule.getXml());
		if (!independentRuleChecker.isIndependent(parsedRule)) {
			LOGGER.error("The rule is not independent. Cannot commit rule.");
			return false;
		}
		return ruleService.commitRule(rule);
	}

	/**
	 * Fetches a rule from the server
	 * 
	 * @param id
	 *            of the rule
	 * @return The rule dto
	 * @throws MalformedURLException
	 */
	public RuleDTO getRule(long id) throws MalformedURLException {
		init();
		return ruleService.getRule(id);
	}

	public URL getUrl() {
		return url;
	}

	public void setUrl(URL url) {
		this.url = url;
	}

}
